@@ -13,7 +13,7 @@ from guardian.shortcuts import get_objects_for_user
 from material import Fieldset, Layout, Row
 
 from aleksis.apps.chronos.managers import TimetableType
-from aleksis.apps.chronos.models import Subject, TimePeriod
+from aleksis.apps.chronos.models import LessonPeriod, Subject, TimePeriod
 from aleksis.core.forms import ActionForm, ListActionForm
 from aleksis.core.models import Group, Person, SchoolTerm
 from aleksis.core.util.core_helpers import get_site_preferences
@@ -45,6 +45,31 @@ class LessonDocumentationForm(forms.ModelForm):
         super().__init__(*args, **kwargs)
 
         self.fields["homework"].label = _("Homework for the next lesson")
+        if (
+            self.instance.lesson_period
+            and get_site_preferences()["alsijil__allow_carry_over_same_week"]
+        ):
+            self.fields["carry_over_week"] = forms.BooleanField(
+                label=_("Carry over data to all other lessons with the same subject in this week"),
+                initial=True,
+                required=False,
+            )
+
+    def save(self, **kwargs):
+        lesson_documentation = super(LessonDocumentationForm, self).save(commit=True)
+        if (
+            get_site_preferences()["alsijil__allow_carry_over_same_week"]
+            and self.cleaned_data["carry_over_week"]
+            and (
+                lesson_documentation.topic
+                or lesson_documentation.homework
+                or lesson_documentation.group_note
+            )
+            and lesson_documentation.lesson_period
+        ):
+            lesson_documentation.carry_over_data(
+                LessonPeriod.objects.filter(lesson=lesson_documentation.lesson_period.lesson)
+            )
 
 
 class PersonalNoteForm(forms.ModelForm):
@@ -111,13 +136,21 @@ class SelectForm(forms.Form):
         if not check_global_permission(self.request.user, "alsijil.view_week"):
             # 1) All groups the user is allowed to see the week view by object permissions
             # 2) All groups the user is a member of an owner of
+            # 3) If the corresponding preference is turned on:
+            # All groups that have a parent group the user is an owner of
             group_qs = (
                 group_qs.filter(
                     pk__in=get_objects_for_user(
                         self.request.user, "core.view_week_class_register_group", Group
                     ).values_list("pk", flat=True)
                 )
-            ).union(group_qs.filter(Q(members=person) | Q(owners=person)))
+            ).union(
+                group_qs.filter(
+                    Q(members=person) | Q(owners=person) | Q(parent_groups__owners=person)
+                    if get_site_preferences()["alsijil__inherit_privileges_from_parent_group"]
+                    else Q(members=person) | Q(owners=person)
+                )
+            )
 
         # Flatten query by filtering groups by pk
         self.fields["group"].queryset = Group.objects.filter(
@@ -130,9 +163,18 @@ class SelectForm(forms.Form):
 
         # Filter selectable teachers by permissions
         if not check_global_permission(self.request.user, "alsijil.view_week"):
-            # If the user hasn't the global permission,
-            # the user is only allowed to see his own person
-            teacher_qs = teacher_qs.filter(pk=person.pk)
+            # If the user hasn't got the global permission and the inherit privileges preference is
+            # turned off, the user is only allowed to see their own person. Otherwise, the user
+            # is allowed to see all persons that teach lessons that the given groups attend.
+            if get_site_preferences()["alsijil__inherit_privileges_from_parent_group"]:
+                teacher_pks = []
+                for group in group_qs:
+                    for lesson in group.lessons.all():
+                        for teacher in lesson.teachers.all():
+                            teacher_pks.append(teacher.pk)
+                teacher_qs = teacher_qs.filter(pk__in=teacher_pks)
+            else:
+                teacher_qs = teacher_qs.filter(pk=person.pk)
 
         self.fields["teacher"].queryset = teacher_qs
 
@@ -144,9 +186,11 @@ PersonalNoteFormSet = forms.modelformset_factory(
 
 class RegisterAbsenceForm(forms.Form):
     layout = Layout(
+        Fieldset("", "person"),
         Fieldset("", Row("date_start", "date_end"), Row("from_period", "to_period")),
         Fieldset("", Row("absent", "excused"), Row("excuse_type"), Row("remarks")),
     )
+    person = forms.ModelChoiceField(label=_("Person"), queryset=None, widget=Select2Widget)
     date_start = forms.DateField(label=_("Start date"), initial=datetime.today)
     date_end = forms.DateField(label=_("End date"), initial=datetime.today)
     from_period = forms.ChoiceField(label=_("Start period"))
@@ -161,10 +205,30 @@ class RegisterAbsenceForm(forms.Form):
     )
     remarks = forms.CharField(label=_("Remarks"), max_length=30, required=False)
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, request, *args, **kwargs):
+        self.request = request
         super().__init__(*args, **kwargs)
         period_choices = TimePeriod.period_choices
 
+        if self.request.user.has_perm("alsijil.register_absence"):
+            self.fields["person"].queryset = Person.objects.all()
+        else:
+            persons_qs = Person.objects.filter(
+                Q(
+                    pk__in=get_objects_for_user(
+                        self.request.user, "core.register_absence_person", Person
+                    )
+                )
+                | Q(primary_group__owners=self.request.user.person)
+                | Q(
+                    member_of__in=get_objects_for_user(
+                        self.request.user, "core.register_absence_group", Group
+                    )
+                )
+            ).distinct()
+
+            self.fields["person"].queryset = persons_qs
+
         self.fields["from_period"].choices = period_choices
         self.fields["to_period"].choices = period_choices
         self.fields["from_period"].initial = TimePeriod.period_min
@@ -180,11 +244,11 @@ class ExtraMarkForm(forms.ModelForm):
 
 
 class ExcuseTypeForm(forms.ModelForm):
-    layout = Layout("short_name", "name")
+    layout = Layout("short_name", "name", "count_as_absent")
 
     class Meta:
         model = ExcuseType
-        fields = ["short_name", "name"]
+        fields = ["short_name", "name", "count_as_absent"]
 
 
 class PersonOverviewForm(ActionForm):
@@ -270,7 +334,12 @@ class AssignGroupRoleForm(forms.ModelForm):
             if "groups" not in initial:
                 groups = (
                     Group.objects.for_current_school_term_or_all()
-                    .filter(owners=self.request.user.person)
+                    .filter(
+                        Q(owners=self.request.user.person)
+                        | Q(parent_groups__owners=self.request.user.person)
+                        if get_site_preferences()["alsijil__inherit_privileges_from_parent_group"]
+                        else Q(owners=self.request.user.person)
+                    )
                     .distinct()
                 )
                 self.fields["groups"].queryset = groups
