@@ -11,6 +11,7 @@ from calendarweek import CalendarWeek
 from aleksis.apps.alsijil.managers import PersonalNoteQuerySet
 from aleksis.apps.chronos.models import Event, ExtraLesson, LessonPeriod
 from aleksis.core.models import Group, Person
+from aleksis.core.util.core_helpers import get_site_preferences
 
 from .models import ExcuseType, ExtraMark, LessonDocumentation, PersonalNote
 
@@ -385,8 +386,16 @@ def get_groups_with_lessons(cls: Group):
 def get_owner_groups_with_lessons(self: Person):
     """Get all groups the person is an owner of and which have related lessons.
 
-    Groups which have child groups with related lessons are also included.
+    Groups which have child groups with related lessons are also included, as well as all
+    child groups of the groups owned by the person with related lessons if the
+    inherit_privileges_from_parent_group preference is turned on.
     """
+    if get_site_preferences()["alsijil__inherit_privileges_from_parent_group"]:
+        return (
+            Group.get_groups_with_lessons()
+            .filter(Q(owners=self) | Q(parent_groups__owners=self))
+            .distinct()
+        )
     return Group.get_groups_with_lessons().filter(owners=self).distinct()
 
 
@@ -423,24 +432,36 @@ def generate_person_list_with_class_register_statistics(
     ).annotate(
         absences_count=Count(
             "filtered_personal_notes",
-            filter=Q(filtered_personal_notes__absent=True),
+            filter=Q(filtered_personal_notes__absent=True)
+            & ~Q(filtered_personal_notes__excuse_type__count_as_absent=False),
+            distinct=True,
         ),
         excused=Count(
+            "filtered_personal_notes",
+            filter=Q(
+                filtered_personal_notes__absent=True,
+                filtered_personal_notes__excused=True,
+            )
+            & ~Q(filtered_personal_notes__excuse_type__count_as_absent=False),
+            distinct=True,
+        ),
+        excused_without_excuse_type=Count(
             "filtered_personal_notes",
             filter=Q(
                 filtered_personal_notes__absent=True,
                 filtered_personal_notes__excused=True,
                 filtered_personal_notes__excuse_type__isnull=True,
             ),
+            distinct=True,
         ),
         unexcused=Count(
             "filtered_personal_notes",
             filter=Q(filtered_personal_notes__absent=True, filtered_personal_notes__excused=False),
+            distinct=True,
         ),
         tardiness=Sum("filtered_personal_notes__late"),
         tardiness_count=Count(
-            "filtered_personal_notes",
-            filter=Q(filtered_personal_notes__late__gt=0),
+            "filtered_personal_notes", filter=Q(filtered_personal_notes__late__gt=0), distinct=True
         ),
     )
 
