@@ -50,6 +50,15 @@ class ExcuseType(ExtensibleModel):
     short_name = models.CharField(max_length=255, unique=True, verbose_name=_("Short name"))
     name = models.CharField(max_length=255, unique=True, verbose_name=_("Name"))
 
+    count_as_absent = models.BooleanField(
+        default=True,
+        verbose_name=_("Count as absent"),
+        help_text=_(
+            "If checked, this excuse type will be counted as a missed lesson. If not checked,"
+            "it won't show up in the absence report."
+        ),
+    )
+
     def __str__(self):
         return f"{self.name} ({self.short_name})"
 
@@ -109,6 +118,14 @@ class RegisterObjectRelatedMixin(WeekRelatedMixin):
         else:
             return self.extra_lesson
 
+    @property
+    def register_object_key(self: Union["LessonDocumentation", "PersonalNote"]) -> str:
+        """Get a unique reference to the related object related."""
+        if self.week and self.year:
+            return f"{self.register_object.pk}_{self.week}_{self.year}"
+        else:
+            return self.register_object.pk
+
     @property
     def calendar_week(self: Union["LessonDocumentation", "PersonalNote"]) -> CalendarWeek:
         """Get the calendar week of this lesson documentation or personal note.
@@ -328,15 +345,13 @@ class LessonDocumentation(RegisterObjectRelatedMixin, ExtensibleModel):
     homework = models.CharField(verbose_name=_("Homework"), max_length=200, blank=True)
     group_note = models.CharField(verbose_name=_("Group note"), max_length=200, blank=True)
 
-    def _carry_over_data(self):
-        """Carry over data to directly adjacent periods in this lesson if data is not already set.
+    def carry_over_data(self, all_periods_of_lesson: LessonPeriod):
+        """Carry over data to given periods in this lesson if data is not already set.
 
-        Can be deactivated using site preference ``alsijil__carry_over``.
+        Both forms of carrying over data can be deactivated using site preferences
+        ``alsijil__carry_over_next_periods`` and ``alsijil__allow_carry_over_same_week``
+        respectively.
         """
-        all_periods_of_lesson = LessonPeriod.objects.filter(
-            lesson=self.lesson_period.lesson,
-            period__weekday=self.lesson_period.period.weekday,
-        )
         for period in all_periods_of_lesson:
             lesson_documentation = period.get_or_create_lesson_documentation(
                 CalendarWeek(week=self.week, year=self.year)
@@ -357,19 +372,24 @@ class LessonDocumentation(RegisterObjectRelatedMixin, ExtensibleModel):
                 changed = True
 
             if changed:
-                lesson_documentation.save(carry_over=False)
+                lesson_documentation.save(carry_over_next_periods=False)
 
     def __str__(self) -> str:
         return f"{self.lesson_period}, {self.date_formatted}"
 
-    def save(self, carry_over=True, *args, **kwargs):
+    def save(self, carry_over_next_periods=True, *args, **kwargs):
         if (
-            get_site_preferences()["alsijil__carry_over"]
+            get_site_preferences()["alsijil__carry_over_next_periods"]
             and (self.topic or self.homework or self.group_note)
             and self.lesson_period
-            and carry_over
+            and carry_over_next_periods
         ):
-            self._carry_over_data()
+            self.carry_over_data(
+                LessonPeriod.objects.filter(
+                    lesson=self.lesson_period.lesson,
+                    period__weekday=self.lesson_period.period.weekday,
+                )
+            )
         super().save(*args, **kwargs)
 
     class Meta:
