@@ -3,6 +3,7 @@ from copy import deepcopy
 from datetime import date, datetime, timedelta
 from typing import Any, Dict, Optional
 
+from django.apps import apps
 from django.core.exceptions import PermissionDenied
 from django.db.models import Count, Exists, FilteredRelation, OuterRef, Prefetch, Q, Sum
 from django.db.models.expressions import Case, When
@@ -185,6 +186,11 @@ def register_object(
     )
 
     if not blocked_because_holidays:
+        groups = register_object.get_groups().all()
+        if groups:
+            first_group = groups.first()
+            context["first_group"] = first_group
+
         # Group roles
         show_group_roles = request.user.person.preferences[
             "alsijil__group_roles_in_lesson_view"
@@ -192,10 +198,22 @@ def register_object(
             "alsijil.view_assigned_grouproles_for_register_object_rule", register_object
         )
         if show_group_roles:
-            groups = register_object.get_groups().all()
             group_roles = GroupRole.objects.with_assignments(date_of_lesson, groups)
             context["group_roles"] = group_roles
 
+        with_seating_plan = (
+            apps.is_installed("aleksis.apps.stoelindeling")
+            and groups
+            and request.user.has_perm("stoelindeling.view_seatingplan_for_group_rule", first_group)
+        )
+        context["with_seating_plan"] = with_seating_plan
+
+        if with_seating_plan:
+            seating_plan = register_object.seating_plan
+            context["seating_plan"] = register_object.seating_plan
+            if seating_plan and seating_plan.group != first_group:
+                context["seating_plan_parent"] = True
+
         # Create or get lesson documentation object; can be empty when first opening lesson
         lesson_documentation = register_object.get_or_create_lesson_documentation(wanted_week)
         context["has_documentation"] = bool(lesson_documentation.topic)
@@ -222,7 +240,11 @@ def register_object(
         else:
             persons = Person.objects.all()
 
-        persons_qs = register_object.get_personal_notes(persons, wanted_week)
+        persons_qs = (
+            register_object.get_personal_notes(persons, wanted_week)
+            .filter(person__member_of__in=request.user.person.owner_of.all())
+            .distinct()
+        )
 
         # Annotate group roles
         if show_group_roles:
@@ -350,9 +372,30 @@ def week_view(
 
     elif hasattr(request, "user") and hasattr(request.user, "person"):
         if request.user.person.lessons_as_teacher.exists():
-            lesson_periods = lesson_periods.filter_teacher(request.user.person)
-            events = events.filter_teacher(request.user.person)
-            extra_lessons = extra_lessons.filter_teacher(request.user.person)
+            inherit_privileges_preference = get_site_preferences()[
+                "alsijil__inherit_privileges_from_parent_group"
+            ]
+            lesson_periods = (
+                lesson_periods.filter_teacher(request.user.person).union(
+                    lesson_periods.filter_groups(request.user.person.owner_of.all())
+                )
+                if inherit_privileges_preference
+                else lesson_periods.filter_teacher(request.user.person)
+            )
+            events = (
+                events.filter_teacher(request.user.person).union(
+                    events.filter_groups(request.user.person.owner_of.all())
+                )
+                if inherit_privileges_preference
+                else events.filter_teacher(request.user.person)
+            )
+            extra_lessons = (
+                extra_lessons.filter_teacher(request.user.person).union(
+                    extra_lessons.filter_groups(request.user.person.owner_of.all())
+                )
+                if inherit_privileges_preference
+                else extra_lessons.filter_teacher(request.user.person)
+            )
 
             type_ = TimetableType.TEACHER
         else:
@@ -435,9 +478,17 @@ def week_view(
         if not request.user.has_perm("alsijil.view_week_personalnote_rule", instance):
             persons_qs = persons_qs.filter(pk=request.user.person.pk)
         elif group:
-            persons_qs = persons_qs.filter(member_of=group)
+            persons_qs = (
+                persons_qs.filter(member_of=group)
+                .filter(member_of__in=request.user.person.owner_of.all())
+                .distinct()
+            )
         else:
-            persons_qs = persons_qs.filter(member_of__in=groups)
+            persons_qs = (
+                persons_qs.filter(member_of__in=groups)
+                .filter(member_of__in=request.user.person.owner_of.all())
+                .distinct()
+            )
 
         # Prefetch object permissions for persons and groups the persons are members of
         # because the object permissions are checked for both persons and groups
@@ -604,16 +655,27 @@ def full_register_group(request: HttpRequest, id_: int) -> HttpResponse:
         | Q(event__groups__parent_groups=group)
     )
     personal_notes = (
-        PersonalNote.objects.select_related("lesson_period")
-        .prefetch_related(
-            "lesson_period__substitutions", "lesson_period__lesson__teachers", "groups_of_person"
+        PersonalNote.objects.prefetch_related(
+            "lesson_period__substitutions", "lesson_period__lesson__teachers"
         )
         .not_empty()
         .filter(groups_q)
+        .filter(groups_of_person=group)
     )
-    documentations = (
-        LessonDocumentation.objects.select_related("lesson_period").not_empty().filter(groups_q)
-    )
+    documentations = LessonDocumentation.objects.not_empty().filter(groups_q)
+
+    sorted_documentations = {"extra_lesson": {}, "event": {}, "lesson_period": {}}
+    sorted_personal_notes = {"extra_lesson": {}, "event": {}, "lesson_period": {}, "person": {}}
+    for documentation in documentations:
+        key = documentation.register_object.label_
+        sorted_documentations[key][documentation.register_object_key] = documentation
+
+    for note in personal_notes:
+        key = note.register_object.label_
+        sorted_personal_notes[key].setdefault(note.register_object_key, [])
+        sorted_personal_notes[key][note.register_object_key].append(note)
+        sorted_personal_notes["person"].setdefault(note.person.pk, [])
+        sorted_personal_notes["person"][note.person.pk].append(note)
 
     # Get all lesson periods for the selected group
     lesson_periods = LessonPeriod.objects.filter_group(group).distinct()
@@ -627,8 +689,8 @@ def full_register_group(request: HttpRequest, id_: int) -> HttpResponse:
         register_objects_by_day.setdefault(day, []).append(
             (
                 extra_lesson,
-                list(filter(lambda d: d.extra_lesson == extra_lesson, documentations)),
-                list(filter(lambda d: d.extra_lesson == extra_lesson, personal_notes)),
+                sorted_documentations["extra_lesson"].get(extra_lesson.pk),
+                sorted_personal_notes["extra_lesson"].get(extra_lesson.pk, []),
                 None,
             )
         )
@@ -642,8 +704,8 @@ def full_register_group(request: HttpRequest, id_: int) -> HttpResponse:
             register_objects_by_day.setdefault(day, []).append(
                 (
                     event_copy,
-                    list(filter(lambda d: d.event == event, documentations)),
-                    list(filter(lambda d: d.event == event, personal_notes)),
+                    sorted_documentations["event"].get(event.pk),
+                    sorted_personal_notes["event"].get(event.pk, []),
                     None,
                 )
             )
@@ -662,26 +724,17 @@ def full_register_group(request: HttpRequest, id_: int) -> HttpResponse:
                 <= day
                 <= lesson_period.lesson.validity.date_end
             ):
-                filtered_documentations = list(
-                    filter(
-                        lambda d: d.week == week.week
-                        and d.year == week.year
-                        and d.lesson_period == lesson_period,
-                        documentations,
-                    )
+                filtered_documentation = sorted_documentations["lesson_period"].get(
+                    f"{lesson_period.pk}_{week.week}_{week.year}"
                 )
-                filtered_personal_notes = list(
-                    filter(
-                        lambda d: d.week == week.week
-                        and d.year == week.year
-                        and d.lesson_period == lesson_period,
-                        personal_notes,
-                    )
+                filtered_personal_notes = sorted_personal_notes["lesson_period"].get(
+                    f"{lesson_period.pk}_{week.week}_{week.year}", []
                 )
+
                 substitution = lesson_period.get_substitution(week)
 
                 register_objects_by_day.setdefault(day, []).append(
-                    (lesson_period, filtered_documentations, filtered_personal_notes, substitution)
+                    (lesson_period, filtered_documentation, filtered_personal_notes, substitution)
                 )
 
     persons = group.members.prefetch_related(None).select_related(None)
@@ -689,12 +742,13 @@ def full_register_group(request: HttpRequest, id_: int) -> HttpResponse:
 
     prefetched_persons = []
     for person in persons:
-        person.filtered_notes = list(filter(lambda d: d.person == person, personal_notes))
+        person.filtered_notes = sorted_personal_notes["person"].get(person.pk, [])
         prefetched_persons.append(person)
 
     context["school_term"] = group.school_term
     context["persons"] = prefetched_persons
-    context["excuse_types"] = ExcuseType.objects.all()
+    context["excuse_types"] = ExcuseType.objects.filter(count_as_absent=True)
+    context["excuse_types_not_absent"] = ExcuseType.objects.filter(count_as_absent=False)
     context["extra_marks"] = ExtraMark.objects.all()
     context["group"] = group
     context["weeks"] = weeks
@@ -703,15 +757,22 @@ def full_register_group(request: HttpRequest, id_: int) -> HttpResponse:
     context["today"] = date.today()
     context["lessons"] = (
         group.lessons.all()
+        .select_related(None)
+        .prefetch_related(None)
         .select_related("validity", "subject")
         .prefetch_related("teachers", "lesson_periods")
     )
-    context["child_groups"] = group.child_groups.all().prefetch_related(
-        "lessons",
-        "lessons__validity",
-        "lessons__subject",
-        "lessons__teachers",
-        "lessons__lesson_periods",
+    context["child_groups"] = (
+        group.child_groups.all()
+        .select_related(None)
+        .prefetch_related(None)
+        .prefetch_related(
+            "lessons",
+            "lessons__validity",
+            "lessons__subject",
+            "lessons__teachers",
+            "lessons__lesson_periods",
+        )
     )
     return render_pdf(request, "alsijil/print/full_register.html", context)
 
@@ -737,11 +798,15 @@ def my_students(request: HttpRequest) -> HttpResponse:
 
     new_groups = []
     for group in relevant_groups:
-        persons = group.generate_person_list_with_class_register_statistics(
-            group.members.prefetch_related(
-                "primary_group__owners",
-                Prefetch("member_of", queryset=relevant_groups, to_attr="member_of_prefetched"),
+        persons = (
+            group.generate_person_list_with_class_register_statistics(
+                group.members.prefetch_related(
+                    "primary_group__owners",
+                    Prefetch("member_of", queryset=relevant_groups, to_attr="member_of_prefetched"),
+                )
             )
+            .filter(member_of__in=request.user.person.owner_of.all())
+            .distinct()
         )
         persons_for_group = []
         for person in persons:
@@ -750,7 +815,8 @@ def my_students(request: HttpRequest) -> HttpResponse:
         new_groups.append((group, persons_for_group))
 
     context["groups"] = new_groups
-    context["excuse_types"] = ExcuseType.objects.all()
+    context["excuse_types"] = ExcuseType.objects.filter(count_as_absent=True)
+    context["excuse_types_not_absent"] = ExcuseType.objects.filter(count_as_absent=False)
     context["extra_marks"] = ExtraMark.objects.all()
     return render(request, "alsijil/class_register/persons.html", context)
 
@@ -774,9 +840,14 @@ class StudentsList(PermissionRequiredMixin, DetailView):
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         context["group"] = self.object
-        context["persons"] = self.object.generate_person_list_with_class_register_statistics()
+        context["persons"] = (
+            self.object.generate_person_list_with_class_register_statistics()
+            .filter(member_of__in=self.request.user.person.owner_of.all())
+            .distinct()
+        )
         context["extra_marks"] = ExtraMark.objects.all()
-        context["excuse_types"] = ExcuseType.objects.all()
+        context["excuse_types"] = ExcuseType.objects.filter(count_as_absent=True)
+        context["excuse_types_not_absent"] = ExcuseType.objects.filter(count_as_absent=False)
         return context
 
 
@@ -893,7 +964,8 @@ def overview_person(request: HttpRequest, id_: Optional[int] = None) -> HttpResp
         note.set_object_permission_checker(checker)
         personal_notes_list.append(note)
     context["personal_notes"] = personal_notes_list
-    context["excuse_types"] = ExcuseType.objects.all()
+    context["excuse_types"] = ExcuseType.objects.filter(count_as_absent=True)
+    context["excuse_types_not_absent"] = ExcuseType.objects.filter(count_as_absent=False)
 
     form = PersonOverviewForm(request, request.POST or None, queryset=allowed_personal_notes)
     if request.method == "POST" and request.user.has_perm(
@@ -927,12 +999,19 @@ def overview_person(request: HttpRequest, id_: Optional[int] = None) -> HttpResp
                 continue
 
             stat.update(
-                personal_notes.filter(absent=True).aggregate(absences_count=Count("absent"))
+                personal_notes.filter(absent=True)
+                .exclude(excuse_type__count_as_absent=False)
+                .aggregate(absences_count=Count("absent"))
             )
             stat.update(
-                personal_notes.filter(
-                    absent=True, excused=True, excuse_type__isnull=True
-                ).aggregate(excused=Count("absent"))
+                personal_notes.filter(absent=True, excused=True)
+                .exclude(excuse_type__count_as_absent=False)
+                .aggregate(excused=Count("absent"))
+            )
+            stat.update(
+                personal_notes.filter(absent=True, excused=True, excuse_type__isnull=True)
+                .exclude(excuse_type__count_as_absent=False)
+                .aggregate(excused_no_excuse_type=Count("absent"))
             )
             stat.update(
                 personal_notes.filter(absent=True, excused=False).aggregate(
@@ -959,7 +1038,6 @@ def overview_person(request: HttpRequest, id_: Optional[int] = None) -> HttpResp
             stats.append((school_term, stat))
         context["stats"] = stats
 
-    context["excuse_types"] = excuse_types
     context["extra_marks"] = extra_marks
 
     # Build filter with own form and logic as django-filter can't work with different models
@@ -992,18 +1070,27 @@ def overview_person(request: HttpRequest, id_: Optional[int] = None) -> HttpResp
 
 @never_cache
 @permission_required("alsijil.register_absence_rule", fn=objectgetter_optional(Person))
-def register_absence(request: HttpRequest, id_: int) -> HttpResponse:
+def register_absence(request: HttpRequest, id_: int = None) -> HttpResponse:
     context = {}
 
-    person = get_object_or_404(Person, pk=id_)
+    if id_:
+        person = get_object_or_404(Person, pk=id_)
+    else:
+        person = None
 
-    register_absence_form = RegisterAbsenceForm(request.POST or None)
+    register_absence_form = RegisterAbsenceForm(
+        request, request.POST or None, initial={"person": person}
+    )
 
-    if request.method == "POST" and register_absence_form.is_valid():
+    if (
+        request.method == "POST"
+        and register_absence_form.is_valid()
+        and request.user.has_perm("alsijil.register_absence_rule", person)
+    ):
         confirmed = request.POST.get("confirmed", "0") == "1"
 
         # Get data from form
-        # person = register_absence_form.cleaned_data["person"]
+        person = register_absence_form.cleaned_data["person"]
         start_date = register_absence_form.cleaned_data["date_start"]
         end_date = register_absence_form.cleaned_data["date_end"]
         from_period = register_absence_form.cleaned_data["from_period"]
